do return end
local MAJOR = "StarLibPluginDBM-1.0" 
local MINOR = 13

local PluginDBM = LibStub:NewLibrary(MAJOR, MINOR)
if not PluginDBM then return end

local _G = _G

if not PluginDBM.__index then
	PluginDBM.__index = PluginDBM
end

local hooks = {}
local function hook(mod, method, func)
	hooks[method] = mod[method]
	mod[method] = function(...)
		mod[method](...)
		func(...)
	end
end

local function remove_hook(mod, method)
	mod[method] = hooks[method]
end

local timer

-- @name PluginDBM:New
-- @usage PluginDBM:New(environment)
-- @parma environment This will be the environment when loadstring is callled.
-- @return A new PluginDBM object
function PluginDBM:New(environment)
	
	local obj = {}

	setmetatable(obj, self)

	if DBM then
		obj:InitializeDBM()
	elseif BigWigs then
		local mod = BigWigs:GetBossModule(module)
				
		local function NewBoss(module, ...)
			local mod = BigWigs:GetBossModule(module) 
			hook(mod, "DelayedMessage", function(key, delay, text, ...) tinsert(history, {text = text, time = GetTime()}) end)         
		end
		
		StarTip:SecureHook(BigWigs, "NewBoss", NewBoss)
	end
	
	return obj		
end

local anchorText = {
	"Top",
	"Top-right",
	"Top-left",
	"Bottom",
	"Bottom-right",
	"Bottom-left",
	"Left",
	"Right",
	"Single Tooltip"
}

local defaults = {
	profile = {
		delay = 3,
		hide = true,
		position = #anchorText,
		onctrl = true
	},
}

local options = {
	hide = {
		name = "Hide DeadlyAnnounce",
		desc = "Toggle whether to hide DeadlyAnnounce after a delay",
		type = "toggle",
		get = function()
			return mod.db.profile.hide
		end,
		set = function(info, v)
			mod.db.profile.hide = v
		end,
		order = 5
	},
	delay = {
		name = "Hide Delay",
		desc = "Enter the time to delay before hiding DeadlyAnnounce",
		type = "input",
		get = function()
			return mod.db.profile.delay
		end,
		set = function(info, v)
			mod.db.profile.delay = v
		end,
		pattern = "%d",
		order = 6
	},
	position = {
		name = "Position",
		desc = "Select where to place tooltip.",
		type = "select",
		values = anchorText,
		get = function() return mod.db.profile.position end,
		set = function(info, v) mod.db.profile.position = v end,
		order = 7
	},
	onctrl = {
		name = "Hide On Ctrl",
		desc = "Whether to hide DeadlyAnnounce when you press the CTRL key.",
		type = "toggle",
		get = function() return mod.db.profile.onctrl end,
		set = function(info, v) mod.db.profile.onctrl = true end,
		order = 8
	}
}

local history = {}

-- Borrowed from BosModTTS
function PluginDBM:InitializeDBM()
	local sound = nil
	local timer = nil
	local text = nil
   
	local function ShowAnnounce(t)
		local new = StarTip.new()
		new.text = t.text
		new.time = _G.GetTime()
		tinsert(history, new)
	end
   
	local function NewAnnounce(announce, _, spellId, ...)
	if announce == nil then
		local spellName = spellId
		text = self.localization.warnings[spellId]
	else
		local spellName = GetSpellInfo(spellId) or "unknown"
         
		if announce == "move" or announce == "you" or announce == "warningspell" then
			if announce == "warningspell" then
				announce = "spell"
			end
           
			text = DBM_CORE_AUTO_SPEC_WARN_TEXTS[announce]:format(spellName)
            
		else
            local spellHaste = select(7, GetSpellInfo(53142)) / 10000 -- 53142 = Dalaran Portal, should have 10000 ms cast time
            local timer = (select(7, GetSpellInfo(spellId)) or 1000) / spellHaste
         
            text = DBM_CORE_AUTO_ANNOUNCE_TEXTS[announce]:format(spellName, (timer / 1000))		
		end
	end
	
	end
      
	local function HookAnnounce(boss)      
		local mod = DBM:GetModByName(boss)
		local announces = mod.announces
      
		for i=1, #announces do
			StarTip:Hook(announces[i], "Show", ShowAnnounce)
		end
	end
		
   
	local function NewMod(_, boss, ...)
		bosses[boss] = true
		local mod = DBM:GetModByName(boss)
      
		self.localization = DBM:GetModLocalization(boss)
      
		hook(mod, "NewTargetAnnounce", function(...) NewAnnounce("target", ...) end)
		hook(mod, "NewSpellAnnounce", function(...) NewAnnounce("spell", ...) end)
		hook(mod, "NewCastAnnounce", function(...) NewAnnounce("cast", ...) end)
		hook(mod, "NewAnnounce", function(...) NewAnnounce(nil, ...) end)
		hook(mod, "NewSpecialWarningMove", function(...) NewAnnounce("move", ...) end)
		hook(mod, "NewSpecialWarningYou", function(...) NewAnnounce("you", ...) end)
		hook(mod, "NewSpecialWarningSpell", function(...) NewAnnounce("warningspell", ...) end)
      
		timer = LibTimer:New("PluginDBM", 1000, false, function() HookAnnounce(boss) end)
	end 

	StarTip:SecureHook(DBM, "NewMod", NewMod)
end

function PluginDBM:Start()
	timer:Start()
	self:InitializeDBM()
end

function PluginDBM:Stop()
	timer:Stop()
	for _, boss in pairs(bosses) do
		local mod = DBM:GetModByName(boss)
		unhook(mod, "NewTargetAnnounce")
		unhook(mod, "NewSpellAnnounce")
		unhook(mod, "NewCastAnnounce")
		unhook(mod, "NewAnnounce")
		unhook(mod, "NewSpecialWarningMove")
		unhook(mod, "NewSpecialWarningYou")
		unhook(mod, "NewSpecialWarningSpell")
	end
	table.wipe(bosses)
end